<?php

namespace HCM\BTL\Controllers;

use Zeedhi\Framework\Controller\Crud;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Request\Row;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO\Request;
use HCM\Util\HCMParameters;
use HCM\Util\DateUtil;

class FasesProcessoController extends Crud {

    protected $dataSourceName = 'des_fase';
    protected $dataSourceName2 = 'dadosParamFases';
    protected $dataSourceName3 = 'getFasesProcesso';
    protected $dataSourceName4 = 'getMinNrFase';

    protected $parameterBagColumns = array('NRORG', 'NRPROCESSO');

    public function __construct (Manager $dataSourceManager, ParameterBag $parameterBag, HCMParameters $hcmParameters){
        $this->parameterBag = $parameterBag;
        $this->hcmParameters   = $hcmParameters;
        parent::__construct($dataSourceManager);
    }

    public function getNrFase (DTO\Request\Filter $request, DTO\Response $response) {
        try{
            $filterData = $request->getFilterCriteria();
            $filterData->setDataSourceName($this->dataSourceName4);
            $newFilterData = $this->populateParameterBag($filterData);
            $dataSet = $this->dataSourceManager->findBy($newFilterData); 
            $response->addDataSet($dataSet);
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function buscarDados (DTO\Request\Filter $request, DTO\Response $response) {
        try{
            $filterData = $request->getFilterCriteria();
            $filterData->setDataSourceName($this->dataSourceName3);
            $newFilterData = $this->populateParameterBag($filterData);
            $dataSet = $this->dataSourceManager->findBy($newFilterData); 
            $buscar = array();
            foreach($dataSet->getRows() as &$row) {
                if(array_key_exists('NRDURACAO', $row)){
                    $row['NRDURACAO'] = DateUtil::getHoraStringDeFloat($row['NRDURACAO']);
                }
                $buscar[] = $row;
            }
            $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('getFasesProcesso', $buscar));
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }


    protected function processConditionValue($condition) {
        return $condition['value'];
    }
    protected function populateParameterBag(FilterCriteria $filterCriteria) {

        $conditions = array();
        foreach ($filterCriteria->getConditions() as $condition) {
            $columnName = $condition['columnName'];
            if (in_array($columnName, $this->parameterBagColumns)) {
                $this->parameterBag->set($columnName, $this->processConditionValue($condition));
            } else {
                $conditions[] = $condition;
            }
        }

        $this->parameterBag->set('NRORG', $this->hcmParameters->getNrorgTrabalho());  
    

        $newFilterCriteria = new FilterCriteria(
            $filterCriteria->getDataSourceName(),
            $conditions,
            $filterCriteria->getPage(),
            $filterCriteria->getPageSize()
        );


        return $newFilterCriteria;
    }

    public function salvarDadosFase (DTO\Request\DataSet $request, DTO\Response $response) {
        try{
            foreach($request->getDataSet()->getRows() as &$row) {
                if(array_key_exists('NRDURACAO', $row)){
                    $row['NRDURACAO'] = DateUtil::getHoraFloatDeString($row['NRDURACAO']);
                }

                if(!array_key_exists('NRCRITERIORESUL', $row)){
                    $row['NRCRITERIORESUL'] = null;
                }


                if($row['__is_new'] === true){
                    if(!array_key_exists('DTINICIOFASE', $row)){
                        $row['DTINICIOFASE'] = null;
                    }

                    if(!array_key_exists('DTFIMFASE', $row)){
                        $row['DTFIMFASE'] = null;
                    }

                    $row['DTINCLUSAO'] = DateUtil::getDataAtual()->format(DateUtil::FORMATO_BRASILEIRO);
                    $row['NRORGINCLUSAO'] = $row['NRORG'];
                    $row['CDOPERINCLUSAO'] = $row['CDOPERADOR_LOG'];
                    
                    $row['NRFASE'] = \HCM\Util\NovoCodigo::geraCodigo($row['NRORG'], 'DES_FASE', 1, 12, true);
                } 

                $row['DTULTATU'] = DateUtil::getDataAtual()->format(DateUtil::FORMATO_BRASILEIRO);
                $row['NRORGULTATU'] = $row['NRORG'];
                $row['CDOPERULTATU'] = $row['CDOPERADOR_LOG'];

            }

            parent::save($request, $response);
            self::salvarDadosParamFase($request, $response);
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function salvarDadosParamFase (DTO\Request\DataSet $request, DTO\Response $response) {
        try{
            $aux = $this->dataSourceName;
            $this->dataSourceName = $this->dataSourceName2;
            foreach($request->getDataSet()->getRows() as &$row) {
                if(array_key_exists('NRPARAMFASE', $row)){
                    if($row['NRPARAMFASE'] === ''){
                        $row['NRPARAMFASE'] = \HCM\Util\NovoCodigo::geraCodigo($row['NRORG'], 'DES_PARAMFASE', 1, 12, true);
                    }
                } else {
                    $row['NRPARAMFASE'] = \HCM\Util\NovoCodigo::geraCodigo($row['NRORG'], 'DES_PARAMFASE', 1, 12, true);
                }
            }
            parent::save($request, $response);
            $this->dataSourceName = $aux;
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function delete(DTO\Request\DataSet $request, DTO\Response $response){
        try{
            foreach($request->getDataSet()->getRows() as &$row) {
                $row['NRORG'] = $this->hcmParameters->getNrorgTrabalho();
            }
            $aux = $this->dataSourceName;
            $this->dataSourceName = $this->dataSourceName2;
            parent::delete($request, $response);
            $this->dataSourceName = $aux;
            parent::delete($request, $response);
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
}