<?php

namespace HCM\FPA\Controllers;

use Zeedhi\Framework\Controller\Crud;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DTO\Response\Notification;
use HCM\Util\HCMParameters;
use Doctrine\DBAL\Connection;
use Doctrine\ORM\EntityManager;
use HCM\Util\DateUtil;

class RateioLancamentoVinculoController extends Crud {

    protected $dataSourceName = 'fpa_rateiovinc';
    protected $dataSourceName1 = 'fpa_itrateiovinc_get';
    
    protected $parameterBagColumns = array('P_NRORG', 'P_NRORG_PADRAO', 'P_DTMESCOMPETENC');
    public function __construct (Manager $dataSourceManager, ParameterBag $parameterBag, HCMParameters $hcmParameters, Connection $connection, EntityManager $entityManager){
        $this->parameterBag = $parameterBag;
        $this->hcmParameters = $hcmParameters;
        $this->connection = $connection;
        $this->entityManager = $entityManager;
        parent::__construct($dataSourceManager);
    }

    public function getRateioLancamentoVinculo(DTO\Request\Filter $request, DTO\Response $response) {
        $filterData = $request->getFilterCriteria();
        $filterData->setDataSourceName($this->dataSourceName);
        $newFilterData = $this->populateParameterBag($filterData);
        $dataSet = $this->dataSourceManager->findBy($newFilterData);
        $response->addDataSet($dataSet);
    }
    
    public function getItemRateio(DTO\Request\Filter $request, DTO\Response $response) {
        $filterData = $request->getFilterCriteria();
        $filterData->setDataSourceName($this->dataSourceName1);
        $newFilterData = $this->populateParameterBag($filterData);
        $dataSet = $this->dataSourceManager->findBy($newFilterData);
        $response->addDataSet($dataSet);
    }
    
    public function setRateioVinc(DTO\Request\DataSet $request, DTO\Response $response) {
        try {
            foreach ($request->getDataSet()->getRows() as &$row) {
                $existeRateio = null;
                $existeRateio = $this->buscaRateioExistente($row['NRORG'], $row['DTINIVIGENCIA']);
                
                if( !$existeRateio ){
                    if (array_key_exists('NRRATEIOVINC', $row)) {
                        if ($row['NRRATEIOVINC'] === '') {
                            $row['NRRATEIOVINC'] = \HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'FPA_RATEIOVINC', 1, 12, true); 
                        }
                    } else {
                        $row['NRRATEIOVINC'] = \HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'FPA_RATEIOVINC', 1, 12, true); 
                    }
                    parent::save($request, $response);
                } else {
                    $response->addNotification(new Notification("Já existe rateio com essa data de vigência", Notification::TYPE_ALERT));
                }
            }
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function setItemRateio(DTO\Request\DataSet $request, DTO\Response $response) {
        try {
            $this->dataSourceName = "fpa_itrateiovinc";
            foreach ($request->getDataSet()->getRows() as &$row) {
                $mesmaEstruturaVinculo = null;
                $mesmaEstruturaVinculo = $this->buscaMesmaEstruturaVinculo($row['NRORG'], $row['NRRATEIOVINC'], $row['NRVINCULOM'], $row['NRESTRUTURAM']);
                
                if( !$mesmaEstruturaVinculo ){
                    if (array_key_exists('NRITRATEIOVINC', $row)) {
                        if ($row['NRITRATEIOVINC'] === '') {
                            $row['NRITRATEIOVINC'] = \HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'FPA_ITRATEIOVINC', 1, 12, true); 
                        }
                    } else {
                        $row['NRITRATEIOVINC'] = \HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'FPA_ITRATEIOVINC', 1, 12, true);
                    }
                    parent::save($request, $response);
                } else {
                    $response->addNotification(new Notification("Já existe item com esse vínculo e com essa estrutura.", Notification::TYPE_ALERT));
                }
            }
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function deleteRateioVinc(DTO\Request\DataSet $request, DTO\Response $response) {
        try{
            parent::delete($request, $response);
        } catch(\Exception $e){
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }    
    }
    
    public function deleteItemRateio(DTO\Request\DataSet $request, DTO\Response $response) {
        try{
            $this->dataSourceName = "fpa_itrateiovinc";
            parent::delete($request, $response);
        } catch(\Exception $e){
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        } 
    }
    
    public function buscaRateioExistente($nrorg, $inicioVigencia) {
        $sql = $this->entityManager->getConnection()->prepare(
        "SELECT * FROM FPA_RATEIOVINC WHERE (NRORG = :P_NRORG AND DTINIVIGENCIA = :P_DTINIVIGENCIA)"
        );
        
        $sql->execute(
            array(
                'P_NRORG' => $nrorg,
                'P_DTINIVIGENCIA' => $inicioVigencia
            )
        );
        
        return $sql->fetchAll();
    }
    
    public function buscaMesmaEstruturaVinculo($nrorg, $nrrateiovinc, $nrvinculom, $nrestruturam) {
        $sql = $this->entityManager->getConnection()->prepare(
        "SELECT * FROM FPA_ITRATEIOVINC WHERE (NRORG = :P_NRORG AND NRVINCULOM = :P_NRVINCULOM AND NRESTRUTURAM = :P_NRESTRUTURAM)"
        );
        
        $sql->execute(
            array(
                'P_NRORG' => $nrorg,
                'P_NRVINCULOM' => $nrvinculom,
                'P_NRESTRUTURAM' => $nrestruturam
            )
        );
        
        return $sql->fetchAll();
    }

    protected function processConditionValue($condition) {
        return $condition['value'];
    }
    protected function populateParameterBag(FilterCriteria $filterCriteria) {

        $conditions = array();
        foreach ($filterCriteria->getConditions() as $condition) {
            $columnName = $condition['columnName'];
            if (in_array($columnName, $this->parameterBagColumns)) {
                $this->parameterBag->set($columnName, $this->processConditionValue($condition));
            } else {
                $conditions[] = $condition;
            }
        }
        $newFilterCriteria = new FilterCriteria(
            $filterCriteria->getDataSourceName(),
            $conditions,
            $filterCriteria->getPage(),
            $filterCriteria->getPageSize()
        );
        return $newFilterCriteria;
    }
} 