<?php

namespace HCM\FPA\Entity;

use HCM\Util\DateUtil;
use HCM\Util\Repositories;
use HCM\Util\TkEntity;
use HCM\Util\MsgCalculo;
use HCM\Geral\Entity\GpesitufunchEntity;
use HCM\Geral\Entity\GpesitufuncmEntity;
use HCM\Util\TkDI;
use Teknisa\Model\Gestaopessoas\Entities\FpaEventom;

class FpaferiasEntity extends TkEntity
{
    private $_nrOrg;
    private $_nrOrgPadrao;
    private $_mesCompetencia;
    private $_gozosFerias;

    private $_situacao_funcional; 
    private $_falta_intervalo; 
    private $_falta_ferias;
    private $_tipo_ferias;
    private $_tipo_feriasEntity;
    private $_nrTpFeriasAnterior;
    private $_periodo_fechado; 
    private $_data_base;
    private $_nova_data_base;
    private $_vai_perder_periodo;
    private $_qtd_meses_periodo_aquisitivo;
    private $_dias_limite_afastamento;
    private $_nrVinculo;
    private $_nrorg;

    const IDCONTROLFERIAS_ABERTO = 'ABERTO';
    const IDCONTROLFERIAS_ADQUIRIDO = 'ADQUIRIDO';
    const IDCONTROLFERIAS_PERDIDO = 'PERDIDO';
    const IDCONTROLFERIAS_GOZADO_PARCIAL = 'GOZADO_PARCIAL';
    const IDCONTROLFERIAS_GOZADO_INTEGRAL = 'GOZADO_INTEGRAL';
    const GOZA_NO_PERIODO_SIM = 'S';
    const GOZA_NO_PERIODO_NAO = 'N';

    public function __construct($entityManager, $hcmParameters){
        $this->hcmParameters = $hcmParameters;
        parent::__construct(null,$entityManager);

        $this->_nrOrg = $this->hcmParameters->getNrorgTrabalho();
        $this->_nrOrgPadrao = $this->hcmParameters->getNrorgPadrao();
        $this->_mesCompetencia = DateUtil::getDataDeString($this->hcmParameters->getDtCompetencia(), DateUtil::FORMATO_BRASILEIRO, true);
        
        $this->_nrTpFeriasAnterior = -1;
        $this->_qtd_meses_periodo_aquisitivo = 0;
        $this->_dias_limite_afastamento = 0;
        $this->_tipo_ferias = new FpatipoferiasmEntity($entityManager, $hcmParameters);
        $this->_falta_ferias = new FpafaltaferiasEntity($entityManager, $hcmParameters);
        $this->_tipo_feriasEntity = $this->_tipo_ferias;
        $this->_situacao_funcional = new GpesitufuncmEntity($entityManager, $hcmParameters);
        
        $this->ferias = Repositories::FPA_FERIAS;
    }

    public function retornaFerias($nr_org, $nr_vinculo){
        $this->resetItems();

        $resultado = $this->entityManager->getRepository(Repositories::FPA_FERIAS)->findBy(
            array('nrorg' => $nr_org, 'nrvinculom' => $nr_vinculo),
            array('idcontrolferias' => 'DESC'));
            
        if (count($resultado) > 0) {
            foreach ($resultado as $entity_obj) {
                $this->addItem($entity_obj);
            }
        }
    }

    public function retornaFeriasPeriodo($nrorg, $nrvinculo, $dt_inicial, $dt_final){
        $this->resetItems();
        
        $query = $this->entityManager->getConnection()->prepare("
            SELECT
                FE.NRVINCULOM,
                GFE.*
            FROM
                FPA_FERIAS FE
                JOIN FPA_GOZOFERIAS GFE ON GFE.NRFERIAS = FE.NRFERIAS
                                        AND GFE.NRORG = FE.NRORG
            WHERE 
               FE.NRVINCULOM = :P_NRVINCULOM
               AND GFE.DTINIGOZOFERIAS = :P_DTINICIOGOZO
               AND GFE.DTFIMGOZOFERIAS = :P_DTFIMGOZO
               AND FE.NRORG = :P_NRORG
        ");
        
        
        $query->execute(
            array(
                "P_NRORG"           => $nrorg,
                "P_NRVINCULOM"      => $nrvinculo,
                "P_DTINICIOGOZO"    => $dt_inicial->format('d/m/Y'),
                "P_DTFIMGOZO"       => $dt_final->format('d/m/Y')
            )
        );

        $resultado = $query->fetchAll(); 
            
        if (count($resultado) > 0) {
            return $resultado;
        }
    }
    
    public function deletaGozoFerias($nrorg, $nrvinculo, $dt_inicial, $dt_final) {
       
        $query = $this->entityManager->getConnection()->prepare("
            DELETE FROM 
              FPA_GOZOFERIAS GFE
            WHERE GFE.NRORG            = :P_NRORG
              AND GFE.DTINIGOZOFERIAS  = :P_DTINICIOGOZO
              AND GFE.DTFIMGOZOFERIAS  = :P_DTFIMGOZO
              AND GFE.NRFERIAS IN (SELECT 
                                     FE.NRFERIAS
                                   FROM 
                                     FPA_FERIAS FE
                                   WHERE 
                                     FE.NRORG          = GFE.NRORG
                                     AND FE.NRVINCULOM = :P_NRVINCULOM)
        ");
        
       $query->execute(
            array(
                "P_NRORG"           => $nrorg,
                "P_NRVINCULOM"      => $nrvinculo,
                "P_DTINICIOGOZO"    => $dt_inicial->format('d/m/Y'),
                "P_DTFIMGOZO"       => $dt_final->format('d/m/Y')
            )
        );
    }
    
    public function resetaFeriasPrevista($nrorg, $nrferias){
        
        $query = $this->entityManager->getConnection()->prepare("
            UPDATE FPA_FERIAS SET DTINIPREVISAO = NULL, 
                      QTDIASPREVISAO = NULL, 
                      DTFINPREVISAO = NULL, 
                      QTABONOPREVISAO=NULL, 
                      DTINIPREVISAO2=NULL,
                      QTDIASPREVISAO2=NULL,
                      DTFINPREVISAO2=NULL,
                      QTABONOPREVISAO2=NULL,
                      DTINIPREVISAO3=NULL,
                      QTDIASPREVISAO3=NULL,
                      DTFINPREVISAO3=NULL,
                      QTABONOPREVISAO3=NULL
            WHERE NRFERIAS = :P_NRFERIAS 
                    AND NRORG = :P_NRORG
        ");
        
        
        $query->execute(
            array(
                "P_NRORG"         => $nrorg,
                "P_NRFERIAS"      => $nrferias
            )
        );

    }

    public function retornaFeriasPorData($dt_busca_ferias){

        foreach ($this->getItems() as $item) {
            if ($dt_busca_ferias->isBetween($item->getDtiniaquisicao(),$item->getDtfimaquisicao())) {
                return $item;
            }
        }
        return false;
    }

    public function retornaFeriasProvisao($nr_org, $nr_vinculo, $dtultimodiames){
        $this->resetItems();

        $this->_nrVinculo = $nr_vinculo;
        $this->_nrorg = $nr_org;
        
        $resultado = $this->entityManager->getRepository(Repositories::FPA_FERIAS)->retornaFeriasProvisao($nr_vinculo, $nr_org, $dtultimodiames);
        
        if (count($resultado) > 0) {
            foreach ($resultado as $entity_obj) {
                $this->addItem($entity_obj);
            }
        }
    }

    public function procuraFerias($nr_ferias = null){
        $itens = $this->getItems();
        if(is_null($nr_ferias)){
            foreach ($itens as $indice => $item) {
                $id_control_fer = $item->getIdcontrolferias();
                if ($id_control_fer == self::IDCONTROLFERIAS_ADQUIRIDO
                    || $id_control_fer == self::IDCONTROLFERIAS_ABERTO
                    || $id_control_fer == self::IDCONTROLFERIAS_GOZADO_PARCIAL) {
                    return $item;
                }
            }
        }
        else{
            foreach ($itens as $indice => $item) {
                $nr_ferias_item = $item->getNrFerias();
                if ((int)$nr_ferias_item === (int)$nr_ferias) {
                    return $item;
                }
            }
            MsgCalculo::getInstance()->adicionaMensagem('Periodo aquisitivo de ferias nao encontrado: '.$nr_ferias);
        }
    }

    public function procuraFeriasColetivas($nr_ferias){
        $itens = $this->getItems();
        
        if(is_null($nr_ferias)){
            sort($itens,0);
            foreach ($itens as $indice => $item) {
                $id_control_fer = $item->getIdcontrolferias();
                if ($id_control_fer == FpaferiasEntity::IDCONTROLFERIAS_ADQUIRIDO
                    || $id_control_fer == FpaferiasEntity::IDCONTROLFERIAS_GOZADO_PARCIAL
                    || $id_control_fer == FpaferiasEntity::IDCONTROLFERIAS_GOZADO_INTEGRAL) {
                    return $item;
                }
            }
        }
        MsgCalculo::getInstance()->adicionaMensagem('Periodo aquisitivo de ferias nao encontrado: '.$nr_ferias);
           
    }

    private function _retornaPeriodoFeriasSeExiste($nr_tipoferias, $data_ini_per) {
        $itens = $this->getItems();
        foreach ($itens as $item) {
            $nr_tpferias = $item->getNrtipoferias();
            $dt_inicio_aquisicao = $item->getDtiniaquisicao();
            if (((int)$nr_tpferias === (int)$nr_tipoferias)
                && ($dt_inicio_aquisicao->format('d/m/Y') === $data_ini_per->format('d/m/Y'))
            ) {
                return $item;
            }
        }

        return false;
    }

    private function verificaPeriodoAberto($vinculo, $dtIniaquisicao) {

        $ferias = \Folha\Entities\FpaFerias::findOneByNrvinculomAndIdcontrolferiasAndNrorg($vinculo->getNrvinculom(),FpaferiasEntity::IDCONTROLFERIAS_ABERTO,$vinculo->getNrorg());
        if($ferias){
            return $ferias;
        }
        
        if($dtIniaquisicao){
          $ferias = \Folha\Entities\FpaFerias::findOneByNrvinculomAndDtiniaquisicaoAndNrorg($vinculo->getNrvinculom(), $dtIniaquisicao, $vinculo->getNrorg());
        }
        
        if($ferias){
          return $ferias;
        }

        return false;
    }
    
    private function ajustaDataBaseFerias($vinculo){
        $dataBase = $vinculo->historico->getDtbaseferias();
        $ferias = $this->procuraFerias();
        
        if($ferias){
            $dtIni = $ferias->getDtiniaquisicao();
            
            if($dataBase > $dtIni){
                $vinculo->historico->setDtbaseferias($dtIni);
                $this->entityManager->persist($vinculo->historico);
                $this->entityManager->flush($vinculo->historico);
            }
        }
    }
    
    private function _determinaDataBaseFerias($vinculo){
        $dataBase = $vinculo->historico->getDtbaseferias();

        if (empty($dataBase)) {
            $dataBase = $vinculo->getDtadmissaovinc();
        }
        return $dataBase;
    }

    public function verificaFeriasRescisao(
        $vinculo, $data_rescisao_projetada, $primeira_vez,
        $evento, $faltasIntervalo, $jornadaDiaria,$nrTipoModalidadeCalc, $nrOcorrenciaCalc,
        $alteracao_situacao_funcional,$mesCompetencia,$qthsemamal,$tipocalculo
    ){

        if ($primeira_vez) {
            
            $FpaTpferiasvincEntity = new FpaTpferiasvincEntity($this->entityManager, $this->hcmParameters);
            $objTpFeriasVinc = $FpaTpferiasvincEntity->retornaTipoFeriasPorTipoVinculo(
                $this->_nrOrg, $this->_nrOrgPadrao, $vinculo->getNrtipovinculom(), $qthsemamal
            );

            $this->retornaFerias($vinculo->getNrorg(), $vinculo->getNrvinculom());

            if($tipocalculo != FpatipocalculoEntity::CALC_RESCISAO_COMPLEMENTAR){
                $this->verificaPeriodoAquisitivo(
                    $vinculo, $evento, $alteracao_situacao_funcional,$faltasIntervalo, $objTpFeriasVinc,
                    $mesCompetencia, $jornadaDiaria,$nrTipoModalidadeCalc, $nrOcorrenciaCalc
                );
            }
        }

        $ferias = array();
        $i = 0;
        
        $_alteracao_situacao_funcional = TkDI::getContainer()->alteracao_situacao_funcional;
        
        $_alteracao_situacao_funcional->retornaAlteracaoSituacaoFuncional(
            $vinculo->getNrorg(), $vinculo->getNrvinculom(), $vinculo->getDtadmissaovinc()
        );
        
        foreach ($this->getItems() as $item) {
		
            $inicio_periodo_aquisitivo = DateUtil::getDataDeString($item->getDtiniaquisicao());
            $inicio_periodo_aquisitivo_aux = DateUtil::getDataDeString($inicio_periodo_aquisitivo);
            if ((DateUtil::setDay($inicio_periodo_aquisitivo_aux, 1)->format('d/m/Y') == $inicio_periodo_aquisitivo->format('d/m/Y')) === false) {
                $inicio_periodo_aquisitivo = DateUtil::setDay($inicio_periodo_aquisitivo, 1);
            }

            $fim_periodo_aquisitivo = DateUtil::getDataDeString($item->getDtfimaquisicao());
            $fim_periodo_aquisitivo_aux = DateUtil::getDataDeString($fim_periodo_aquisitivo);
            
            $fim_periodo_aquisitivo_aux = DateUtil::getUltimoDiaDoMes($fim_periodo_aquisitivo_aux);
            if (($fim_periodo_aquisitivo_aux->format('d/m/Y') == $fim_periodo_aquisitivo->format('d/m/Y')) === false) {
                $fim_periodo_aquisitivo = DateUtil::SubtraiIntervalo(DateUtil::getUltimoDiaDoMes($fim_periodo_aquisitivo_aux), 1, DateUtil::MES);
            }
		
            $diasGozados=0;
            if (($item->getIdcontrolferias() === FpaferiasEntity::IDCONTROLFERIAS_GOZADO_PARCIAL)
                ||($item->getIdcontrolferias() === FpaferiasEntity::IDCONTROLFERIAS_ADQUIRIDO)
            ) {
                $data_ini_per_aux = DateUtil::getDataDeString($item->getDtiniaquisicao());
                
                $qtd_doze_avos = 0;
                
                $diasFaltas = $this->_totalizaDiasFaltas(
                    $vinculo, $evento, $faltasIntervalo, $inicio_periodo_aquisitivo,
                    $fim_periodo_aquisitivo, $jornadaDiaria, 1,
                    $nrOcorrenciaCalc
                );
                $falta_ferias = $this->_falta_ferias->verificaFaltaFerias($item->getNrtipoferias(), $diasFaltas);
                if (!$falta_ferias) {
                    $item->setQtdiasmaxfe(0);
                    $item->setQtdiasmaxabon(0);
                    $item->setIdcontrolferias(self::IDCONTROLFERIAS_PERDIDO);
					$qtd_dias = 0;
                } else {
                    $item->setQtdiasmaxfe($falta_ferias->getQtdiasmaxfe());
                    $item->setQtdiasmaxabon($falta_ferias->getQtdiasmaxabon());
                    $qtd_dias = $falta_ferias->getQtdiasmaxfe();					
                }
                
                $data_fim_per = DateUtil::getDataDeString($data_ini_per_aux);
                
                $qtd_doze_avos = DateUtil::getQtdeMesesEntre($data_ini_per_aux, $data_rescisao_projetada) > 0 ? DateUtil::getQtdeMesesEntre($data_ini_per_aux, $data_rescisao_projetada) : 0;
                
                $data_fim_per = DateUtil::adicionaIntervalo($data_fim_per, $qtd_doze_avos, DateUtil::MES);
                if ($data_fim_per->format('d/m/Y') == $data_rescisao_projetada->format('d/m/Y')
                    || $data_fim_per < $data_rescisao_projetada)
                {
                    $dias = DateUtil::getQtdeDiasEntre($data_fim_per, $data_rescisao_projetada) + 1;
                    if ($dias >= 15) {
                        $qtd_doze_avos++;
                    }
                }
                
                $mes_afastados = $this->_quantidadeMesAfastados($data_ini_per_aux, $fim_periodo_aquisitivo);
                
                $qtd_doze_avos -= $mes_afastados;
                $qtd_doze_avos = $qtd_doze_avos < 0 ? 0 : $qtd_doze_avos;
                if($mes_afastados > 0){
                    $diasSuspensos = $mes_afastados * 2.5;
                    $qtd_dias -= $diasSuspensos;
                }
                
                if($qtd_doze_avos == 0){
                    $qtd_dias = 0;
                }
                
                $ferias[$i]['DTFIMAQUISICAO'] = $item->getDtfimaquisicao();
                if($item->getIdcontrolferias() == FpaferiasEntity::IDCONTROLFERIAS_GOZADO_PARCIAL){
                    $diasGozados=0;
                    foreach ($this->getGozoFerias($item->getNrferias()) as $itemGozo) {
                        $diasGozados += $itemGozo->getDiasGozados() + $itemGozo->getDiasAbonados();
                    }
                }
                

            } elseif ($item->getIdcontrolferias() === FpaferiasEntity::IDCONTROLFERIAS_ABERTO) {
                $data_ini_per = DateUtil::getDataDeString($item->getDtiniaquisicao());
                $data_fim_per = DateUtil::getDataDeString($data_ini_per);
                $data_ini_per_aux = $data_ini_per;

                $diasFaltas = $this->_totalizaDiasFaltas(
                    $vinculo, $evento, $faltasIntervalo, $inicio_periodo_aquisitivo,
                    $fim_periodo_aquisitivo, $jornadaDiaria, 1,
                    $nrOcorrenciaCalc
                );
                
                $qtd_doze_avos = DateUtil::getQtdeMesesEntre($data_ini_per, $data_rescisao_projetada);
                $falta_ferias = $this->_falta_ferias->verificaFaltaFerias($item->getNrtipoferias(), $diasFaltas);
                if (!$falta_ferias) {
                    $item->setQtdiasmaxfe(0);
                    $item->setQtdiasmaxabon(0);
                    $qtd_doze_avos = 0;
					$qtd_dias = 0;
                    $item->setIdcontrolferias(self::IDCONTROLFERIAS_PERDIDO);
                } else {
                    $item->setQtdiasmaxfe($falta_ferias->getQtdiasmaxfe());
                    $item->setQtdiasmaxabon($falta_ferias->getQtdiasmaxabon());	
                    $qtd_dias = $falta_ferias->getQtdiasmaxfe();					
                }
                $qtd_doze_avos = $qtd_doze_avos < 0 ? 0 : $qtd_doze_avos;
                
                $data_fim_per = DateUtil::adicionaIntervalo($data_fim_per, $qtd_doze_avos, DateUtil::MES);
                
                if ($data_fim_per->format('d/m/Y') == $data_rescisao_projetada->format('d/m/Y')
                    || $data_fim_per < $data_rescisao_projetada)
                {
                    $dias = DateUtil::getQtdeDiasEntre($data_fim_per, $data_rescisao_projetada) + 1;
                    if ($dias >= 15) {
                        $qtd_doze_avos++;
                    }
                }
                
                $mes_afastados = $this->_quantidadeMesAfastados($data_ini_per, $data_fim_per);
                
                $qtd_doze_avos -= $mes_afastados;
                
                $qtd_doze_avos = $qtd_doze_avos < 0 ? 0 : $qtd_doze_avos;
                
                $ferias[$i]['DTFIMAQUISICAO'] = $data_rescisao_projetada;
            } else {
                continue;
            }
            $ferias[$i]['TIPOFERIAS'] = $item->getNrtipoferias();
            $ferias[$i]['IDCONTROLFERIAS'] = $item->getIdcontrolferias();
            $ferias[$i]['DTINIAQUISICAO'] = $item->getDtiniaquisicao();
            $ferias[$i]['QTDIASMAXFE'] = $qtd_dias;
            $ferias[$i]['QTMESESMAXFE'] = $qtd_doze_avos;
            $ferias[$i]['QTDDIASGOZADOS']= $diasGozados;
            $i++;
        }
        
        return $ferias;
    }
    
    public function _quantidadeMesAfastados($dt_inicio, $dt_fim){
        
        $_alteracao_situacao_funcional = TkDI::getContainer()->alteracao_situacao_funcional;
        $mes_ini = $dt_inicio->format('m');
        $mes_fim = $dt_fim->format('m');
        $dt_aux_inicio = $dt_inicio;
        $mes_afastado = 0;
        
        for ($i = $dt_inicio; $i <= $dt_fim; $i = DateUtil::adicionaIntervalo($i, 1, DateUtil::MES)) {
            
            if ($i == $dt_inicio) {
                $dt_aux_inicio = $dt_inicio;
                $dt_aux_fim = DateUtil::adicionaIntervalo($dt_inicio, 1, DateUtil::MES);
                if($dt_aux_fim > $dt_fim){                          
                    $dt_aux_fim = $dt_fim;      
                }
            } elseif ($i == $dt_fim) {
                $dt_aux_inicio = DateUtil::setDay($dt_fim, 1);
                $dt_aux_fim = $dt_fim;
            }else {
                $dt_aux_inicio = $i;
                $dt_aux_fim = DateUtil::adicionaIntervalo($dt_aux_inicio, 1, DateUtil::MES);
            }
            
            $afastamento = $_alteracao_situacao_funcional->retornaDiasAfastamento(
                $this->_situacao_funcional, $dt_aux_inicio, $dt_aux_fim, false
            );
            $dias_afastados = 0;
            for ($x=0; $x < count($afastamento); $x++) {
                $sit_func = $this->_situacao_funcional->retornaSituacaoFuncional(
                    $this->_nrOrgPadrao, $this->_nrorg, $afastamento[$x]
                );
            
                if ($sit_func == true) {
                    if (($sit_func->getNrsitufuncm() === 100040) or 
                        ($sit_func->getNrsitufuncm() === 100041) or
                        ($sit_func->getNrsitufuncm() === 100044)) {
                        $dias_afastados++;
                    }
                }
            }
            if($dias_afastados > 15){
                $mes_afastado++;
            }
            
        }
        return $mes_afastado;
    }

    public function contabilizaFeriasColetivas(
        $vinculo, $dtIniGozo, boolean $primeira_vez,
        $evento, $faltasIntervalo, $jornadaDiaria,$nrTipoModalidadeCalc, $nrOcorrenciaCalc,
        $alteracao_situacao_funcional,$mesCompetencia,$qthsemamal,$diasGozo
    ){
        if ($primeira_vez) {
            $this->retornaFerias($vinculo->getNrorg(), $vinculo->getNrvinculom());
        }

        $ferias = array();
        $i = 0;
        $data_ini_per = new TkDateTime();
        $data_fim_per = new TkDateTime();
        $goza_neste = FpaFeriasEntityBase::GOZA_NO_PERIODO_NAO;
        $saldo=0;
        $itens=$this->getItems();

        foreach ($itens as $item) {
            $diasGozados=0;
            $diasdireito=0;
            if (($item->getIdcontrolferias() === FpaferiasEntity::IDCONTROLFERIAS_GOZADO_PARCIAL)
                ||($item->getIdcontrolferias() === FpaferiasEntity::IDCONTROLFERIAS_ADQUIRIDO)
            ) {
                $data_ini_per_aux= new TkDateTime($item->getDtiniaquisicao());

                $diasFaltas = $this->_totalizaDiasFaltas(
                    $vinculo, $evento, $faltasIntervalo, $data_ini_per_aux->setDay(1),
                    $item->getDtfimaquisicao(), $jornadaDiaria, $nrTipoModalidadeCalc,
                    $nrOcorrenciaCalc
                );
                $falta_ferias = $this->_falta_ferias->verificaFaltaFerias($item->getNrtipoferias(), $diasFaltas);
                if (!$falta_ferias) {
                    $item->setQtdiasmaxfe(0);
                    $item->setQtdiasmaxabon(0);
                    $item->setIdcontrolferias(self::IDCONTROLFERIAS_PERDIDO);
                    $qtd_dias = 0;
                } else {
                    $item->setQtdiasmaxfe($falta_ferias->getQtdiasmaxfe());
                    $item->setQtdiasmaxabon($falta_ferias->getQtdiasmaxabon());
                    $qtd_dias = $falta_ferias->getQtdiasmaxfe();
                }

                $qtd_doze_avos = 0;
                if($item->getIdcontrolferias() == FpaferiasEntity::IDCONTROLFERIAS_GOZADO_PARCIAL){
                    $diasGozados=0;
                    foreach ($this->getGozoFerias($item->getNrferias()) as $itemGozo) {
                        $diasGozados += $itemGozo->getDiasGozados() + $itemGozo->getDiasAbonados();
                    }
                }
                $diasdireito=$qtd_dias-$diasGozados;
            } elseif ($item->getIdcontrolferias() === FpaferiasEntity::IDCONTROLFERIAS_ABERTO) {
                $data_ini_per->set($item->getDtiniaquisicao(), TkDateTime::FORMAT_DATE);
                $data_fim_per->set($data_ini_per, TkDateTime::FORMAT_DATE);
                
                $data_ini_per_aux = clone $data_ini_per;

                $diasFaltas = $this->_totalizaDiasFaltas(
                    $vinculo, $evento, $faltasIntervalo, $data_ini_per_aux->setDay(1),
                    $item->getDtfimaquisicao(), $jornadaDiaria, $nrTipoModalidadeCalc,
                    $nrOcorrenciaCalc
                );
                $dtIniGozoAux = clone($dtIniGozo);
                $dtIniGozoAux->addDay(-1);
                $qtd_doze_avos = $data_ini_per->months($dtIniGozoAux);
                $falta_ferias = $this->_falta_ferias->verificaFaltaFerias($item->getNrtipoferias(), $diasFaltas);
                if (!$falta_ferias) {
                    $item->setQtdiasmaxfe(0);
                    $item->setQtdiasmaxabon(0);
                    $qtd_doze_avos = 0;
                    $qtd_dias = 0;
                    $item->setIdcontrolferias(self::IDCONTROLFERIAS_PERDIDO);
                } else {
                    $item->setQtdiasmaxfe($falta_ferias->getQtdiasmaxfe());
                    $item->setQtdiasmaxabon($falta_ferias->getQtdiasmaxabon());
                    $qtd_dias = $falta_ferias->getQtdiasmaxfe();
                }
                
                $data_fim_per->addMonth($qtd_doze_avos);
                if (($data_fim_per->equals($dtIniGozoAux, TkDateTime::FORMAT_DATE) === true)
                    ||($data_fim_per->isEarlier($dtIniGozoAux, TkDateTime::FORMAT_DATE) === true)
                ) {
                    $dias = $data_fim_per->daysAfter($dtIniGozoAux);
                    if ($dias >= 15) {
                        $qtd_doze_avos++;
                    }
                }
                $nrTipoFerias=$item->getNrtipoferias();
                $mult = $diasdireito=FpatipoferiasmEntity::retornaDiasDireito($nrTipoFerias);
                
                $diasdireito=ceil($qtd_doze_avos*$mult)-$diasGozados;
            } elseif($item->getIdcontrolferias() === FpaferiasEntity::IDCONTROLFERIAS_PERDIDO) {
                $qtd_dias=0;
                $qtd_doze_avos=0;
                $diasdireito=0;
            }else{
                continue;
            }
            if($goza_neste==FpaFeriasEntityBase::GOZA_NO_PERIODO_NAO && $diasdireito>0){
                $goza_neste=FpaFeriasEntityBase::GOZA_NO_PERIODO_SIM;
            }else{
                $goza_neste=FpaFeriasEntityBase::GOZA_NO_PERIODO_NAO;
            }
            $ferias[$i]['NRFERIAS'] = $item->getNrferias();
            $ferias[$i]['TIPOFERIAS'] = $item->getNrtipoferias();
            $ferias[$i]['IDCONTROLFERIAS'] = $item->getIdcontrolferias();
            $ferias[$i]['DTINIAQUISICAO'] = $item->getDtiniaquisicao();
            $ferias[$i]['DTFIMAQUISICAO'] = $item->getDtfimaquisicao();
            $ferias[$i]['QTDIASMAXFE'] = $qtd_dias;
            $ferias[$i]['QTMESESMAXFE'] = $qtd_doze_avos;
            $ferias[$i]['QTDDIASGOZADOS']= $diasGozados;
            $ferias[$i]['DIASDIREITO']=$diasdireito;
            $ferias[$i]['GOZAPERIODO']=$goza_neste;
            $ferias[$i]['SALDO']=$saldo;
            
            if($goza_neste==FpaFeriasEntityBase::GOZA_NO_PERIODO_SIM && $diasGozo>$diasdireito){
                $saldo=$diasdireito-$diasGozo;
                if(count($ferias)==1 && $item->getIdcontrolferias()==FpaferiasEntity::IDCONTROLFERIAS_ABERTO){
                    $ferias[$i]['SALDO']=0;
                }
            }
            $i++;
        }
        return $ferias;
    }
    
    public function verificaFeriasProvisao(
        $vinculo, $data_rescisao_projetada, $primeira_vez,
        $evento, $faltasIntervalo, $jornadaDiaria,$nrTipoModalidadeCalc, $nrOcorrenciaCalc,
        $alteracao_situacao_funcional,$mesCompetencia,$qthsemamal
    ){

        if ($primeira_vez) {
            $FpaTpferiasvincEntity = new FpaTpferiasvincEntity($this->entityManager, $this->hcmParameters);
            
            $objTpFeriasVinc = $FpaTpferiasvincEntity->retornaTipoFeriasPorTipoVinculo(
                $this->_nrOrg, $this->_nrOrgPadrao, $vinculo->getNrtipovinculom(),$qthsemamal
            );

            $this->retornaFeriasProvisao($vinculo->getNrorg(), $vinculo->getNrvinculom(),$data_rescisao_projetada);

        }

        $ferias = array();
        $i = 0;

        foreach ($this->getItems() as $item) {
            $diasGozados=0;
            $idcontrole=$item->getIdcontrolferias();
            if (($item->getIdcontrolferias() === FpaferiasEntity::IDCONTROLFERIAS_GOZADO_PARCIAL)
                ||($item->getIdcontrolferias() === FpaferiasEntity::IDCONTROLFERIAS_ADQUIRIDO)
            ) {
                
                $data_ini_per_aux= DateUtil::setDay($item->getDtiniaquisicao(), 1);

                $diasFaltas = $this->_totalizaDiasFaltas(
                    $vinculo, $evento, $faltasIntervalo, $data_ini_per_aux,
                    $item->getDtfimaquisicao(), $jornadaDiaria, $nrTipoModalidadeCalc,
                    $nrOcorrenciaCalc
                );
                
                $falta_ferias = $this->_falta_ferias->verificaFaltaFerias($item->getNrtipoferias(), $diasFaltas);
                
                if (!$falta_ferias) {
                    $item->setQtdiasmaxfe(0);
                    $item->setQtdiasmaxabon(0);
                    $item->setIdcontrolferias(self::IDCONTROLFERIAS_PERDIDO);
                    $qtd_dias = 0;
                } else {
                    $item->setQtdiasmaxfe($falta_ferias->getQtdiasmaxfe());
                    $item->setQtdiasmaxabon($falta_ferias->getQtdiasmaxabon());
                    $qtd_dias = $falta_ferias->getQtdiasmaxfe();
                }

                $qtd_doze_avos = 0;
                $ferias[$i]['DTFIMAQUISICAO'] = $item->getDtfimaquisicao();
                $idcontrole=$item->getIdcontrolferias();
                if($item->getIdcontrolferias() == FpaferiasEntity::IDCONTROLFERIAS_GOZADO_PARCIAL){
                    $diasGozados=0;
                    $diasAbonados=0;
                    foreach ($this->getGozoFerias($item->getNrferias()) as $itemGozo) {
                        $dtinigozo = DateUtil::getDataDeString($itemGozo->getDtinigozoferias());
                        if(!($dtinigozo > $data_rescisao_projetada)){
                            $diasGozados += $itemGozo->getDiasGozados();
                            $diasAbonados+= $itemGozo->getDiasAbonados();
                        }
                    }
                    $qtd_dias = is_null($item->getQtdiasmaxfe()) ? 30:$item->getQtdiasmaxfe();
                    if(($diasGozados+$diasAbonados)==0){
                        $idcontrole='ADQUIRIDO';
                    }elseif(($diasGozados+$diasAbonados)<$qtd_dias){
                        $idcontrole='GOZADO_PARCIAL';
                    }
                    //$diasGozados=0;
                }

            } elseif ($item->getIdcontrolferias() === FpaferiasEntity::IDCONTROLFERIAS_ABERTO) {
                $data_ini_per = DateUtil::getDataDeString($item->getDtiniaquisicao());
                $data_fim_per = $data_ini_per;
                $data_ini_per_aux= DateUtil::setDay($data_ini_per, 1);

                $diasFaltas = $this->_totalizaDiasFaltas(
                    $vinculo, $evento, $faltasIntervalo, $data_ini_per_aux,
                    $item->getDtfimaquisicao(), $jornadaDiaria, $nrTipoModalidadeCalc,
                    $nrOcorrenciaCalc
                );
                
                $qtd_doze_avos = DateUtil::getQtdeMesesEntre($data_ini_per, $data_rescisao_projetada);
                
                $falta_ferias = $this->_falta_ferias->verificaFaltaFerias($item->getNrtipoferias(), $diasFaltas);
                if (!$falta_ferias) {
                    $item->setQtdiasmaxfe(0);
                    $item->setQtdiasmaxabon(0);
                    $qtd_doze_avos = 0;
                    $qtd_dias = 0;
                    $item->setIdcontrolferias(self::IDCONTROLFERIAS_PERDIDO);
                } else {
                    $item->setQtdiasmaxfe($falta_ferias->getQtdiasmaxfe());
                    $item->setQtdiasmaxabon($falta_ferias->getQtdiasmaxabon());
                    $qtd_dias = $falta_ferias->getQtdiasmaxfe();
                }
                
                $data_fim_per = DateUtil::adicionaIntervalo($data_fim_per, $qtd_doze_avos, DateUtil::MES);
                
                if ($data_fim_per->format('d/m/Y') === $data_rescisao_projetada->format('d/m/Y')
                    || $data_fim_per < $data_rescisao_projetada
                ) {
                    $dias = DateUtil::getQtdeDiasEntre($data_fim_per, $data_rescisao_projetada) + 1;
                    if ($dias >= 15) {
                        $qtd_doze_avos++;
                    }
                }

                $ferias[$i]['DTFIMAQUISICAO'] = $data_rescisao_projetada;
                $idcontrole=$item->getIdcontrolferias();
            } elseif ($item->getIdcontrolferias() === FpaferiasEntity::IDCONTROLFERIAS_GOZADO_INTEGRAL) {
                $diasGozados=0;
                $diasAbonados=0;
                foreach ($this->getGozoFerias($item->getNrferias()) as $itemGozo) {
                    $dtinigozo = DateUtil::getDataDeString($itemGozo->getDtinigozoferias());
                    if(!($dtinigozo > $data_rescisao_projetada)){
                        $diasGozados += $itemGozo->getDiasGozados();
                        $diasAbonados+= $itemGozo->getDiasAbonados();
                    }
                }
                $qtd_dias = is_null($item->getQtdiasmaxfe()) ? 30:$item->getQtdiasmaxfe();
                $idcontrole=$item->getIdcontrolferias();
                if(($diasGozados+$diasAbonados)>=$qtd_dias){
                    $idcontrole=FpaferiasEntity::IDCONTROLFERIAS_GOZADO_INTEGRAL;
                }elseif(($diasGozados+$diasAbonados)==0){
                    $idcontrole=FpaferiasEntity::IDCONTROLFERIAS_ADQUIRIDO;
                }elseif(($diasGozados+$diasAbonados)<$qtd_dias){
                    $idcontrole=FpaferiasEntity::IDCONTROLFERIAS_GOZADO_PARCIAL;
                }
                $ferias[$i]['DTFIMAQUISICAO'] = $item->getDtfimaquisicao();
            }
            $ferias[$i]['TIPOFERIAS'] = $item->getNrtipoferias();
            $ferias[$i]['IDCONTROLFERIAS'] = $idcontrole;
            $ferias[$i]['DTINIAQUISICAO'] = $item->getDtiniaquisicao();
            $ferias[$i]['QTDIASMAXFE'] = $qtd_dias;
            $ferias[$i]['QTMESESMAXFE'] = $qtd_doze_avos;
            $ferias[$i]['QTDDIASGOZADOS']= $diasGozados;
            
            if($idcontrole == FpaferiasEntity::IDCONTROLFERIAS_GOZADO_INTEGRAL){
                unset($ferias[$i]);
            };
            if($item->getIdcontrolferias() == FpaferiasEntity::IDCONTROLFERIAS_PERDIDO){
                unset($ferias[$i]);
            };
            $i++;
        }
        $ferias=array_values($ferias);
        return $ferias;
    }

    private function _atualizaDataBaseFeriasVinculo(
        $data_ini_per, $data_fim_per, $vinculo
    ){
        if ($this->_vai_perder_periodo === true) {
            if ($this->_nova_data_base->equals(TkDateTime::DUMMY_DATE, TkDateTime::FORMAT_DATE) === false) {
                $data_ini_per->set($vinculo->historico->getDtbaseferias());
                $data_fim_per->set($data_ini_per)->addYear(1)->subDay(1);
            }
        }
    }

    public function verificaPeriodoAquisitivo(
        $vinculo, $evento, $alteracao_situacao_funcional,
        $faltas_intervalo, array $objTpFeriasVinc, $mes_competencia, $jornada_diaria,
        $nrTipoModalidadeCalc, $nrOcorrenciaCalc
    ){
        $this->ajustaDataBaseFerias($vinculo);
        $data_base = $this->_determinaDataBaseFerias($vinculo);
        $this->_data_base = DateUtil::getDataDeString(substr($data_base, 0, 10), DateUtil::FORMATO_BRASILEIRO, true);
        
        foreach ($objTpFeriasVinc as $tpFeriasVinc) {
            $nr_tipoferias = $tpFeriasVinc->getNrtipoferias();
            
            if ($this->_nrTpFeriasAnterior !== $nr_tipoferias) {
                $obj_tipo_ferias = $this->_tipo_feriasEntity->retornaTipoFerias(
                    $this->_nrOrgPadrao, $this->_nrOrg, $nr_tipoferias
                );
                $this->_nrTpFeriasAnterior = $nr_tipoferias;
                $this->_tipo_ferias = $obj_tipo_ferias;
                if (is_object($obj_tipo_ferias) === false) {
                    MsgCalculo::getInstance()->adicionaMensagem('Tipo de férias não encontrado. Verificar Tipo Férias x Tipo Vínculo.');
                    return;
                }
                $this->_qtd_meses_periodo_aquisitivo = $this->_tipo_feriasEntity->retornaQtdMesesPeriodoAquisitivo(
                    $obj_tipo_ferias
                );

                $this->_dias_limite_afastamento = $obj_tipo_ferias->historico->getQtmesafastperda() * 30;
            }

            $data_ini_per = $data_base;
            
            $data_fim_per = DateUtil::adicionaIntervalo($data_ini_per, $this->_qtd_meses_periodo_aquisitivo, DateUtil::MES);
            $data_fim_per = DateUtil::subtraiIntervalo($data_fim_per, 1);
            
            if (DateUtil::getUltimoDiaDoMes($mes_competencia) < $data_ini_per) {
                MsgCalculo::getInstance()->adicionaMensagem('Mês de competencia menor que Data de início do período de férias.');
                return;
            }

            $ferias = $this->_verificaExistenciaPeriodoFerias(
                $mes_competencia, $data_ini_per, $data_fim_per, $vinculo, $evento, $alteracao_situacao_funcional,
                $faltas_intervalo, $jornada_diaria, $this->_tipo_ferias, $nrTipoModalidadeCalc, $nrOcorrenciaCalc
            );
        }

    }

    private function _verificaExistenciaPeriodoFerias(
        $mes_competencia, $data_ini_per, $data_fim_per, $vinculo,
        $evento, $alteracao_situacao_funcional,
        $faltas_intervalo, $jornada_diaria, $obj_tipo_ferias,
        $nrTipoModalidadeCalc, $nrOcorrenciaCalc
    ){
        $achou_periodo_ferias = false;
        $persiste_ferias = false;
        $nr_tipoferias = $obj_tipo_ferias->getNrtipoferias();
        $nr_vinculo = $vinculo->getNrvinculom();
        
        while (!$achou_periodo_ferias) {
            $ferias = $this->_retornaPeriodoFeriasSeExiste($nr_tipoferias, $data_ini_per);
            if ($ferias && ($ferias->getIdcontrolferias() == self::IDCONTROLFERIAS_ABERTO         ||
                            $ferias->getIdcontrolferias() == self::IDCONTROLFERIAS_GOZADO_PARCIAL ||
                            $ferias->getIdcontrolferias() == self::IDCONTROLFERIAS_ADQUIRIDO)
            ) {
                $dt_fim_per_falta = DateUtil::subtraiIntervalo($data_fim_per, 1, DateUtil::MES);
                
                $persiste_aux = $this->_verificaAfastamentosFaltas(
                    $vinculo, $ferias, $evento, $alteracao_situacao_funcional, $faltas_intervalo, $data_ini_per,
                    $dt_fim_per_falta, $jornada_diaria, $nr_tipoferias, $nrTipoModalidadeCalc,
                    $nrOcorrenciaCalc
                );

                if ($persiste_aux === true) {
                    $persiste_ferias = true;
                }
                $diasGozados=0;
                $diasAbono=0;
                
                $diasDireito= $ferias->getQtdiasmaxfe()==null||$ferias->getQtdiasmaxfe()==0?30:$ferias->getQtdiasmaxfe();
                $diasFaltas = $this->_totalizaDiasFaltas(
                    $vinculo, $evento, $faltas_intervalo, $data_ini_per,
                    $data_fim_per, $jornada_diaria, $nrTipoModalidadeCalc,
                    $nrOcorrenciaCalc
                );

                $faltaFerias = $this->_falta_ferias->verificaFaltaFerias($ferias->getNrtipoferias(), $diasFaltas);

                if (is_object($faltaFerias)) {
                    if ($diasDireito != $faltaFerias->getQtdiasmaxfe()) {
                        $diasDireito = $faltaFerias->getQtdiasmaxfe();
                        $ferias->setQtdiasmaxfe($diasDireito);
                    }
                }
                foreach ($this->getGozoFerias($ferias->getNrferias()) as $itemGozo) {
                    $diasGozados += $itemGozo->getDiasGozados();
                    $diasAbono +=(int)$itemGozo->getQtdiasabonoferi();
                }
                if ($diasAbono + $diasGozados > 0) {
                    if ($diasAbono + $diasGozados >= $diasDireito) {
                        $ferias->setIdcontrolferias(self::IDCONTROLFERIAS_GOZADO_INTEGRAL);
                        if ($this->_data_base > $data_ini_per || $this->_data_base->format('d/m/Y') == $data_ini_per->format('d/m/Y')){
                            $vinculo->historico->setDtbaseferias($data_ini_per);
                        };
                    } else {
                        $ferias->setIdcontrolferias(self::IDCONTROLFERIAS_GOZADO_PARCIAL);
                    }
                    
                    $this->entityManager->persist($ferias);
                    $this->entityManager->flush($ferias);
                }
            }
            
            $dt_atual = DateUtil::getDataAtual(true);
            $mes_competencia = DateUtil::getDataDeString(substr($mes_competencia, 0, 10), DateUtil::FORMATO_BRASILEIRO, true);
            
            if ($mes_competencia > $data_fim_per || $dt_atual > $data_fim_per && $dt_atual->format('m') == $mes_competencia->format('m')) {
                if ($ferias && ($ferias->getIdcontrolferias() == self::IDCONTROLFERIAS_ABERTO)) {
                    $ferias->setIdcontrolferias(self::IDCONTROLFERIAS_ADQUIRIDO);
                    $this->entityManager->persist($ferias);
                    $this->entityManager->flush($ferias);
                }
                $data_ini_per = DateUtil::adicionaIntervalo($data_fim_per, 1);
                $data_fim_per = DateUtil::subtraiIntervalo(DateUtil::adicionaIntervalo($data_ini_per, $this->_qtd_meses_periodo_aquisitivo, DateUtil::MES), 1);
            } else {
                if (!$ferias) {
                    if($this->verificaPeriodoAberto($vinculo, $data_ini_per)){
                        MsgCalculo::getInstance()->adicionaMensagem('O vínculo possui período aquisitivo em aberto. Verifique a data base e/ou o tipo de férias');
                        return;
                    };
                    
                    $diasMaxFerias = $obj_tipo_ferias->historico->getQtdiasmaxfe();
                    $diasMaxAbono = $obj_tipo_ferias->historico->getQtdiasmaxabon();
                    
                    $vrReferenciaFerias30 = $this->entityManager->getRepository(Repositories::FPA_VALORREFERENCIAM)->buscavalor($this->_nrorg, $this->_nrOrgPadrao, $mes_competencia, FpaValorreferenciamEntity::FERIAS_SEMPRE_30);    
                    if($vrReferenciaFerias30 == '1'){
                        $diasMaxFerias = 30;
                        $diasMaxAbono = 10;
                        $nr_tipoferias = 1;
                    }
                    
                    $nrferias=$this->incluiFerias(
                        $this->_nrOrg,$nr_vinculo,$data_ini_per ,$data_fim_per,$nr_tipoferias,0, $diasMaxFerias,
                        $diasMaxAbono, 0, self::IDCONTROLFERIAS_ABERTO);

                }
                $achou_periodo_ferias = true;
            }
        }
    }

    public function _verificaAfastamentosFaltas(
        $vinculo, $ferias, $evento, $alteracao_situacao_funcional,
         $faltas_intervalo, $data_ini_per, $data_fim_per, $jornada_diaria,
        $nr_tipoferias, $nrTipoModalidadeCalc, $nrOcorrenciaCalc
    ){
        if($data_fim_per > $this->_mesCompetencia){
            $data_fim_per = DateUtil::getUltimoDiaDoMes($this->_mesCompetencia);
        }
        $persiste = false;
        
        $item_ferias = $ferias;
        if ($item_ferias != false) {
            $dias_faltas = $this->_totalizaDiasFaltas(
                $vinculo, $evento, $faltas_intervalo, $data_ini_per, $data_fim_per, $jornada_diaria,
                $nrTipoModalidadeCalc, $nrOcorrenciaCalc
            );
            
            $dias_afastados = $this->_totalizaDiasAfastamento(
                $alteracao_situacao_funcional, $data_ini_per, $data_fim_per, $this->_dias_limite_afastamento
            );
            
            $meses_afastados = floor($dias_afastados/30);
            
            if (($dias_faltas > 0)
                ||($dias_afastados > 0)
            ) {
                if ($this->_vai_perder_periodo === true) {
                    $item_ferias->setQtdiasmaxfe(0);
                    $item_ferias->setQtdiasmaxabon(0);
                    $item_ferias->setIdcontrolferias(self::IDCONTROLFERIAS_PERDIDO);
                    $item_ferias->setDtultatu(DateUtil::getDataAtual());
                   
                    /*if ($this->_nova_data_base->format('d/m/Y') == DateUtil::getDataAtual(true)->format('d/m/Y')) {
                        $vinculo->historico->atualizaDataBaseFerias($this->_nova_data_base);
                    }*/
                } else {
                    $falta_ferias = $this->_falta_ferias->verificaFaltaFerias($nr_tipoferias, $dias_faltas);
                    if (!$falta_ferias) {
                        $item_ferias->setQtdiasmaxfe(0);
                        $item_ferias->setQtdiasmaxabon(0);

                        $item_ferias->setIdcontrolferias(self::IDCONTROLFERIAS_PERDIDO);
                    } else {
                        $item_ferias->setQtdiasmaxfe($falta_ferias->getQtdiasmaxfe());
                        $item_ferias->setQtdiasmaxabon($falta_ferias->getQtdiasmaxabon());
                    }
                }
                $item_ferias->setQtdiaafastament($dias_afastados);
                $item_ferias->setQtmesafastament($meses_afastados);
                $item_ferias->setQtfaltasperiodo($dias_faltas);
                $persiste = true;
            }
        }
        return $persiste;
    }

    private function _verificaPerdaPeriodo($dias_afastados, $dias_limite){
        if ((int)$dias_afastados >= (int)$dias_limite) {
            $this->_vai_perder_periodo = true;
        }
    }

    public function _totalizaDiasAfastamento(
        $alteracao_situacao_funcional, $data_ini_per,
        $data_fim_per, $dias_limite
    ){
        $data_fim_aux = DateUtil::adicionaIntervalo($data_fim_per, 1);
        
        $afastamento = $alteracao_situacao_funcional->retornaDiasAfastamento(null, $data_ini_per, $data_fim_aux, false);
        $dias_afastados = 0;
        $this->_nova_data_base = DateUtil::getDataAtual(true);
        $this->_vai_perder_periodo = false;
        
        for ($i=0; $i<count($afastamento); $i++) {
            if ($afastamento[$i] !== -1) {
                $sit_func = $this->_situacao_funcional->retornaSituacaoFuncional(
                    $this->_nrOrgPadrao, $this->_nrOrg, $afastamento[$i]
                );
                if ($i < (count($afastamento)-1) && $sit_func) {
                    if ((string)$sit_func->historico->getIdsuspferias() === GpesitufunchEntity::IDSUSPFERIAS_SIM) {
                        $dias_afastados++;
                         $this->_verificaPerdaPeriodo($dias_afastados, $dias_limite);
                    }
                }

                if ($this->_vai_perder_periodo === true && $sit_func) {
                    if ((string)$sit_func->historico->getIdsuspferias() === GpesitufunchEntity::IDSUSPFERIAS_NAO) {
                        $this->_nova_data_base = DateUtil::adicionaIntervalo($data_ini_per, $i);
                    }
                    
                }
            } else {
                if ($this->_vai_perder_periodo === true) {
                        $this->_nova_data_base = DateUtil::adicionaIntervalo($data_ini_per, $i);
                }
            }
        }
        return $dias_afastados;
    }

    private function _totalizaMesesAfastamento(
        $alteracao_situacao_funcional, $data_ini_per, $data_fim_per,
        $dias_afastados
    ) {

        $meses_afastados = $alteracao_situacao_funcional->retornaMesesAfastamento(
            $data_ini_per, $data_fim_per, $dias_afastados
        );
        return $meses_afastados;

    }

    public function _totalizaDiasFaltas(
        $vinculo, $obj_evento,  $faltas_intervalo,
        $data_ini_per, $data_fim_per, $jornada_diaria, $nrTipoModalidadeCalc, $nrOcorrenciaCalc
    ) {
        $lista_eventos = $faltas_intervalo->calcula(
            $vinculo, $data_ini_per, $data_fim_per, $nrTipoModalidadeCalc, $nrOcorrenciaCalc, $this->_nrOrgPadrao
        );
        $total_dias = 0;
        if($lista_eventos){
            foreach ($lista_eventos as $nr_evento=> $item_evento) {
                $evento = $obj_evento->retornaEvento($nr_evento);
                $tipo_referencia = $evento->historico->getIdunidrefereven();
                $id_faltas_ferias = $evento->historico->getIdfaltasferias();
                $total_aux = number_format(
                    FpaEventom::compatibilizaReferenciaDias($item_evento['REF'], $tipo_referencia, $jornada_diaria), 2
                );
        
                switch ($id_faltas_ferias){
                case FpaeventohEntity::IDFALTASFERIAS_SOMA:		$total_dias += $total_aux; break;
                case FpaeventohEntity::IDFALTASFERIAS_SUBTRAI: 	$total_dias -= $total_aux; break;
                }
            }
        }

        return  floor($total_dias);
    }

    public function incluiFerias(
        $nrOrg, $nrVinculom, $dtIniAquisicao, $dtFimAquisicao, $nrTipoFerias, $qtdFaltas, $qtdDiasMaxFerias,
        $qtdDiasMaxAbono, $qtdDiasAfastamento, $idControlFerias
    ){
        $item = new $this->ferias;
        $item->setNrorg($nrOrg);
        $item->setNrvinculom($nrVinculom);
        $item->setDtiniaquisicao($dtIniAquisicao);
        $item->setDtfimaquisicao($dtFimAquisicao);
        $item->setNrtipoferias($nrTipoFerias);
        $item->setQtfaltasperiodo($qtdFaltas);
        $item->setQtdiasmaxfe($qtdDiasMaxFerias);
        $item->setQtdiasmaxabon($qtdDiasMaxAbono);
        $item->setQtdiaafastament($qtdDiasAfastamento);
        $item->setIdcontrolferias($idControlFerias);
        $item->setQtmesafastament(0);
        
        
        $item->setDtinclusao(DateUtil::getDataAtual());
        $item->setNrorginclusao($this->hcmParameters->getNrorgTrabalho());
        $item->setCdoperinclusao($this->hcmParameters->getCdoperador());
        $item->setDtultatu(DateUtil::getDataAtual());
        $item->setNrorgultatu($this->hcmParameters->getNrorgTrabalho());
        $item->setCdoperultatu($this->hcmParameters->getCdoperador());
        
        $item->setNrferias(\HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'FPA_FERIAS', 1, 12, true));
        
        $this->entityManager->persist($item);
        $this->entityManager->flush($item);
        
        $nrFerias = $item->getNrferias();
        unset($item);
        return $nrFerias;
    }

    public function setFerias($nrorg, $nrvinculom, $dtiniaquisicao, $dtfimaquisicao, $nrtipoferias, $qtfaltas,
                                 $qtdiasmaxferias, $qtdiasmaxabono, $qtdiasafastamento, $idcontroleferias)
    {
        $this->setNrorg($nrorg);
        $this->setNrvinculom($nrvinculom);
        $this->setDtiniaquisicao($dtiniaquisicao);
        $this->setDtfimaquisicao($dtfimaquisicao);
        $this->setNrtipoferias($nrtipoferias);
        $this->setQtfaltasperiodo($qtfaltas);
        $this->setQtdiasmaxfe($qtdiasmaxferias);
        $this->setQtdiasmaxabon($qtdiasmaxabono);
        $this->setQtdiaafastament($qtdiasafastamento);
        $this->setIdcontrolferias($idcontroleferias);
    }

    public static function getByNrorgVinculomTipoferiasData($nrorg, $nrvinculom, $tpferias, $dtiniaquisicao)
    {
        $fpaferias = self::getEntityByWhere(
            'NRORG = ? AND NRVINCULOM = ? AND NRTIPOFERIAS = ? AND DTINIAQUISICAO = ?',
            array($nrorg, $nrvinculom, $tpferias, $dtiniaquisicao)
        );
        return $fpaferias;
    }

    public function getGozoFerias($nrFerias) {
        
        $this->_gozosFerias = $this->entityManager->getRepository(Repositories::FPA_GOZOFERIAS)->findBy(
            array('nrorg' => $this->_nrOrg, 'nrferias' => $nrFerias),
            array('dtinigozoferias' => 'ASC'));
    
        return $this->_gozosFerias;
    }
    
    public function getGozoFeriasDatas($nrFerias, $dtIni, $dtFim) {
        
        $this->_gozosFerias = $this->entityManager->getRepository(Repositories::FPA_GOZOFERIAS)->retornaGozoFeriasDataInicio($this->_nrOrg, $nrFerias, $dtIni, $dtFim);
    
        return $this->_gozosFerias;
    }
    
}