<?php
/**
 * Created by PhpStorm.
 * User: felipesantos
 * Date: 18/08/2017
 * Time: 11:09
 */

namespace HCM\Geral\Controllers;

use HCM\Geral\Services\EsoParametroService;
use HCM\Util\HCMParameters;
use Zeedhi\Framework\Controller\Crud;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Request;

class EsoParametroController extends Crud {

    protected $dataSourceName = 'eso_parametro_set';
    protected $dataSourceName_get = 'eso_parametro_get';

    protected $parameterBagColumns = array('P_NRORG', 'P_DTMESCOMPETENC');

    public function __construct (Manager $dataSourceManager, ParameterBag $parameterBag, HCMParameters $hcmParameters, EsoParametroService $esoParametroService){
        $this->parameterBag = $parameterBag;
        $this->hcmParameters = $hcmParameters;
        $this->esoParametroService = $esoParametroService;
        parent::__construct($dataSourceManager);
    }

    public function getEsoParametro (DTO\Request\Filter $request, DTO\Response $response) {
        try {
            $filterData = $request->getFilterCriteria();
            $filterData->setDataSourceName($this->dataSourceName_get);
            $newFilterData = $this->populateParameterBag($filterData);
            $dataSet = $this->dataSourceManager->findBy($newFilterData);
            $response->addDataSet($dataSet);
        } catch(\Exception $e) {
            $response->addMessage(new DTO\Response\Message($e->getMessage(), DTO\Response\Message::TYPE_ERROR));
        }
    }

    public function setEsoParametro(DTO\Request\DataSet $request, DTO\Response $response) {
        try {
            foreach ($request->getDataSet()->getRows() as &$row) {
                $row['IDATIVO'] = isset($row['IDATIVO']) ? $row['IDATIVO'] : 'S';
                $row['NRPARCRESPPERI'] = isset($row['NRPARCRESPPERI']) ? $row['NRPARCRESPPERI'] : null;
                $row['IDUTILIZAMATRIC']= isset($row['IDUTILIZAMATRIC']) ? $row['IDUTILIZAMATRIC'] : 'N';
                
                if (array_key_exists('NRPARAMETRO', $row)) {
                    if ($row['NRPARAMETRO'] === '') {
                        $row['NRPARAMETRO'] = \HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'ESO_PARAMETRO', 1, 12, true);
                    }
                } else {
                    $row['NRPARAMETRO'] = \HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'ESO_PARAMETRO', 1, 12, true);
                }
                
                //Verifica se foi alterada a versão para atualizar os envios pendentes para a nova versão
                $this->esoParametroService->verificaVersaoNova($row);
            }
            parent::save($request, $response);
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function deleteEsoParametro(DTO\Request\DataSet $request, DTO\Response $response){
        try {
            parent::delete($request, $response);
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function setCertTransmissor(DTO\Request\DataSet $request, DTO\Response $response) {
        try {
            foreach ($request->getDataSet()->getRows() as &$row) {
                if (is_null($row['NRANEXO'])) {
                    unset($row['NRANEXO']);
                }
                $upload = $this->esoParametroService->setDadosCertificado($row);
                $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('setCertTransmissor', array($upload)));
            }
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    protected function processConditionValue($condition) {
        return $condition['value'];
    }
    protected function populateParameterBag(FilterCriteria $filterCriteria) {

        $conditions = array();
        foreach ($filterCriteria->getConditions() as $condition) {
            $columnName = $condition['columnName'];
            if (in_array($columnName, $this->parameterBagColumns)) {
                $this->parameterBag->set($columnName, $this->processConditionValue($condition));
            } else {
                $conditions[] = $condition;
            }
        }
        $newFilterCriteria = new FilterCriteria(
            $filterCriteria->getDataSourceName(),
            $conditions,
            $filterCriteria->getPage(),
            $filterCriteria->getPageSize()
        );


        return $newFilterCriteria;
    }
} 