/**
 * Rotina de apurao do faturamento servios a partir do aceite da proposta.
 * Pacote de servios contratado na proposta.
 * 
 * @author Diego Gonalves (17/03/2011) 
 *
 * @param P_CDCLIENTE Cdigo do cliente
 *        P_CDOPERAPUR Cdigo do operador
 *        P_COMMIT Identificador de execuo de commit no final
 *
 * @return void
 */

CREATE OR REPLACE 
PROCEDURE FATURA_SERVICO_ACEITE
   (P_CDCLIENTE IN VENCSERVCLIE.CDCLIENTE%TYPE,
    P_CDOPERAPUR IN OPERADOR.CDOPERADOR%TYPE,
    P_COMMIT IN VARCHAR2 := '1')
AS
   V_PRXAPURACAO VARCHAR2(20);
   V_DSBOLETCLIE VARCHAR2(2000);
   V_DSFATURCLIE VARCHAR2(2000);
   V_CDEMPRESA VARCHAR2(2);
   V_CDFILIAL VARCHAR2(4);
   V_DATA DATE;
   V_MES DATE;
   V_VERIFMES BOOLEAN;
   V_PARCELA NUMBER;
   V_PROPOSTA VARCHAR2(12);
   V_DESCBOLETO VARCHAR2(100);
   V_DSNOTACLIE VARCHAR2(100);
BEGIN
   SELECT TRUNC(SYSDATE)
     INTO V_DATA
     FROM DUAL;

   V_VERIFMES := TRUE;

   V_PARCELA  := 0;
   V_PROPOSTA := 'X';

   /* EXECUTAR O FOR PARA PEGAR AS PARCELAS DO VENCIMENTO ANTES DE GERAR A APURAO */

   FOR CUR_VENC IN (SELECT VENC.CDCLIENTE, VENC.DTVENCSERVCLIE, VENC.VRVENCSERVCLIE, NVL(E.CDPAIS, '0055') CDPAIS, VENC.NRPROPOSFAS, LTRIM(VENC.NRPROPOSFAS,0) AS NRPROPOSTABOL, VENC.NRVERPROPFAS,
                           (SELECT COUNT(*) AS QTDE_PARC
                                          FROM VENCSERVCLIE V
                                         WHERE V.CDCLIENTE = P_CDCLIENTE
                                           AND VENC.NRPROPOSFAS = V.NRPROPOSFAS
                                           AND VENC.NRVERPROPFAS = V.NRVERPROPFAS) TOTAL_PARCELA
                      FROM VENCSERVCLIE VENC, ENDECLIE E
                     WHERE VENC.CDCLIENTE = P_CDCLIENTE
                       AND ((VENC.VRVENCSERVCLIE > 0) OR (VENC.VRVENCSERVCLIE IS NULL))
                       AND VENC.NRAPURFATFAS IS NULL
                       AND E.IDTPENDECLIE = 'P'
                       AND E.CDCLIENTE = P_CDCLIENTE
                     ORDER BY VENC.NRPROPOSFAS, VENC.NRVERPROPFAS, VENC.DTVENCSERVCLIE) LOOP

      V_DESCBOLETO := '';
      V_DSNOTACLIE := '';

      IF (CUR_VENC.NRPROPOSFAS||CUR_VENC.NRVERPROPFAS <> V_PROPOSTA) THEN
         V_PARCELA  := 1;
         V_PROPOSTA := CUR_VENC.NRPROPOSFAS||CUR_VENC.NRVERPROPFAS;
      ELSE
         V_PARCELA := V_PARCELA + 1;
      END IF;
                     
      IF V_VERIFMES THEN
         V_MES := ADD_MONTHS(LAST_DAY(TRUNC(CUR_VENC.DTVENCSERVCLIE)), -1) + 1;
         V_VERIFMES := FALSE;
      END IF;

   BEGIN
         SELECT MAX(P.DSBOLETCLIE), MAX(P.DSFATURCLIE), NVL(MAX(P.CDEMPRESA), '01'), NVL(MAX(F.CDFILIAL), '9999') -- ver se pode retornar mais de uma filial
           INTO V_DSBOLETCLIE, V_DSFATURCLIE, V_CDEMPRESA, V_CDFILIAL
           FROM PARCLIEFAS P, FILIAL F
          WHERE P.IDTPFATUCLIE = '5' -- MO DE OBRA
            AND P.CDCLIENTE = CUR_VENC.CDCLIENTE
            AND P.CDEMPRESA = F.CDEMPRESA
            AND ROWNUM = 1;
      EXCEPTION
         WHEN NO_DATA_FOUND THEN
            NULL;
      END;

      /*A EMPRESA 12 POSSUI 3 FILIAIS, A PEDIDO DA POLIANA SER GERADO O FATURAMENTO SEMPRE PARA A FILIAL 9990*/ 
      IF V_CDEMPRESA = '12' THEN
         V_CDFILIAL := '9990';
      END IF;

      /*V_DESCBOLETO := V_DSBOLETCLIE;*/
      /* conforme acordado com a Poliana,
        a descrio do boleto e da nota sero fixas,
        para que o incluir um novo cliente nao seja necessario alterar a descrio*/
      V_DESCBOLETO := 'Suporte ref proposta COMXXXXXX-XX. Parcela XX/XX.';
      V_DESCBOLETO := REPLACE(V_DESCBOLETO, 'XXXXXX-XX', CUR_VENC.NRPROPOSTABOL||'-'||CUR_VENC.NRVERPROPFAS);

      IF (LPAD(V_PARCELA,2,'0') = '01') AND (LPAD(CUR_VENC.TOTAL_PARCELA,2,'0') = '01') THEN
         V_DESCBOLETO := REPLACE(V_DESCBOLETO, 'XX/XX', 'nica');
      ELSE
         V_DESCBOLETO := REPLACE(V_DESCBOLETO, 'XX/XX', LPAD(V_PARCELA,2,'0')||'/'||LPAD(CUR_VENC.TOTAL_PARCELA,2,'0'));
      END IF;

      /*V_DSNOTACLIE := V_DSFATURCLIE;*/
      V_DSNOTACLIE := 'Prestao de servios de suporte tcnico ref prop COMxxxxxx-x. Parcela XX/XX.';
      V_DSNOTACLIE := REPLACE(V_DSNOTACLIE, 'xxxxxx-x', CUR_VENC.NRPROPOSTABOL||'-'||CUR_VENC.NRVERPROPFAS);
      IF (LPAD(V_PARCELA,2,'0') = '01') AND (LPAD(CUR_VENC.TOTAL_PARCELA,2,'0') = '01') THEN
         V_DSNOTACLIE := REPLACE(V_DSNOTACLIE, 'XX/XX', 'nica');
      ELSE
         V_DSNOTACLIE := REPLACE(V_DSNOTACLIE , 'XX/XX', LPAD(V_PARCELA,2,'0')||'/'||LPAD(CUR_VENC.TOTAL_PARCELA,2,'0'));
      END IF;

      NOVO_CODIGO('APURFATUFAS', V_PRXAPURACAO);

      INSERT INTO APURFATUFAS
         (CDFILIAL, CDCLIENTE, NRAPURFATFAS,
          DTAPURFATFAS, VRTOTAFATFAS, DSEMISSPRONF,
          CDOPERAPUR, CDEMPRESA, CDPRODUTO,
          IDAPURFATFAS, IDSTAPFATFAS, IDCOMIFATFAS,
          DSBOLEFATFAS, DTMESAPURFAS, CDGRUPONFAPUR,
          NRPROPOSFAS, NRVERPROPFAS)
      VALUES
         (V_CDFILIAL, CUR_VENC.CDCLIENTE, SUBSTR(V_PRXAPURACAO, 11, 10),
          V_DATA, CUR_VENC.VRVENCSERVCLIE, V_DSNOTACLIE,
          P_CDOPERAPUR, V_CDEMPRESA, '6102003',
          '1', 'P', 'N',
          V_DESCBOLETO, V_MES, '01',
          CUR_VENC.NRPROPOSFAS, CUR_VENC.NRVERPROPFAS);
          -- POLIANA PEDIU PARA COLOCAR FIXO COM O PRODUTO 'Consultoria', POIS ATUALMENTE
          -- ELA ALTERA MANUALMENTE TODAS AS APURAÇÕES. TAMBÉM CITOU QUE ESTE É O PRODUTO CORRETO
          -- QUE PASSA NA SECRETARIA DA FAZENDA DA PREFEITURA (EDERSON 23/06/2017)

      UPDATE VENCSERVCLIE
         SET NRAPURFATFAS = SUBSTR(V_PRXAPURACAO, 11, 10)
       WHERE CDCLIENTE = CUR_VENC.CDCLIENTE
         AND DTVENCSERVCLIE = CUR_VENC.DTVENCSERVCLIE;

      V_MES := ADD_MONTHS(V_MES, 1);
   END LOOP;
   
   IF P_COMMIT = '1' THEN
      COMMIT;
   END IF;
END;
/

-- VERSAO --
BEGIN ATUVEROBJ('FATURA_SERVICO_ACEITE','5.20.000'); END;
/